#include "rom/miniz.h"

class GzipDecompressor {
private:
    static const size_t CHUNK_SIZE = 1024;  // Adjust based on your needs

public:
    // Returns decompressed size or 0 if failed
    static size_t getDecompressedSize(const uint8_t* input, size_t inputSize) {
        size_t decompSize = 0;
        // Last 4 bytes of gzip contain decompressed size
        if (inputSize >= 4) {
            decompSize = (input[inputSize - 1] << 24) |
                        (input[inputSize - 2] << 16) |
                        (input[inputSize - 3] << 8) |
                        (input[inputSize - 4]);
        }
        return decompSize;
    }

    static bool decompressData(const uint8_t* input, size_t inputSize, 
                             uint8_t* output, size_t* outputSize) {
        tinfl_decompressor decomp;
        tinfl_init(&decomp);

        size_t inPos = 0;
        size_t outPos = 0;
        
        // Skip gzip header (usually 10 bytes)
        if (inputSize > 10) {
            inPos = 10;
        }

        while (inPos < inputSize - 8) {  // -8 for footer
            size_t inBytes = inputSize - inPos - 8;  // Don't read footer
            size_t outBytes = *outputSize - outPos;
            
            tinfl_status status = tinfl_decompress(&decomp,
                &input[inPos], &inBytes,
                output, &output[outPos], &outBytes,
                TINFL_FLAG_HAS_MORE_INPUT | TINFL_FLAG_USING_NON_WRAPPING_OUTPUT_BUF);

            inPos += inBytes;
            outPos += outBytes;

            if (status == TINFL_STATUS_DONE) {
                *outputSize = outPos;
                return true;
            } else if (status < 0) {
                return false;
            }
        }
        return false;
    }
};